'use strict';

var fetch = require('node-fetch');

class StremThruError extends Error {
  body;
  code = "UNKNOWN";
  headers;
  statusCode;
  statusText;
  type = "unknown_error";
  constructor(message, options) {
    super(message);
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this.constructor);
    }
    if (options?.cause) {
      this.cause = options.cause;
      delete options.cause;
    }
    if (options.body) {
      this.body = options.body;
    }
    this.headers = options.headers;
    this.statusCode = options.statusCode;
    this.statusText = options.statusText;
    if (options.type) {
      this.type = options.type;
    }
    if (options.code) {
      this.code = options.code;
    }
  }
}

var version = "0.7.0";

const USER_AGENT = `stremthru:sdk:js/${version}`;
class StremThruStore {
  #client;
  #clientIp;
  constructor(client, clientIp) {
    this.#client = client;
    if (clientIp) {
      this.#clientIp = clientIp;
    }
  }
  async addMagnet({
    clientIp = this.#clientIp,
    magnet
  }) {
    return await this.#client.request("/v0/store/magnets", {
      body: { magnet },
      method: "POST",
      params: clientIp ? { client_ip: clientIp } : {}
    });
  }
  async checkMagnet(params) {
    return await this.#client.request("/v0/store/magnets/check", {
      method: "GET",
      params
    });
  }
  async generateLink({
    clientIp = this.#clientIp,
    link
  }) {
    return await this.#client.request(`/v0/store/link/generate`, {
      body: { link },
      method: "POST",
      params: clientIp ? { client_ip: clientIp } : {}
    });
  }
  async getMagnet(magnetId) {
    return await this.#client.request(`/v0/store/magnets/${magnetId}`, { method: "GET" });
  }
  async getUser() {
    return await this.#client.request("/v0/store/user", { method: "GET" });
  }
  async listMagnets({
    limit,
    offset
  }) {
    const params = {};
    if (limit) {
      params["limit"] = String(limit);
    }
    if (offset) {
      params["offset"] = String(offset);
    }
    return await this.#client.request("/v0/store/magnets", { method: "GET", params });
  }
  async removeMagnet(magnetId) {
    return await this.#client.request(`/v0/store/magnets/${magnetId}`, {
      method: "DELETE"
    });
  }
}
class StremThru {
  baseUrl;
  store;
  #headers;
  #timeout;
  constructor(config) {
    this.baseUrl = config.baseUrl;
    this.#headers = {
      "User-Agent": [USER_AGENT, config.userAgent].filter(Boolean).join(" ")
    };
    if (config.timeout) {
      this.#timeout = config.timeout;
    }
    if (config.auth) {
      if (typeof config.auth === "object" && "user" in config.auth) {
        config.auth = `${config.auth.user}:${config.auth.pass}`;
      }
      if (typeof config.auth === "string") {
        if (config.auth.includes(":")) {
          config.auth = Buffer.from(config.auth.trim()).toString("base64");
        }
        this.#headers["Proxy-Authorization"] = `Basic ${config.auth}`;
      } else if ("store" in config.auth) {
        this.#headers["X-StremThru-Store-Name"] = config.auth.store;
        this.#headers["X-StremThru-Store-Authorization"] = `Bearer ${config.auth.token}`;
      }
    }
    this.store = new StremThruStore(this, config.clientIp);
  }
  async health() {
    return await this.request(`/v0/health`, {});
  }
  async request(endpoint, {
    body,
    headers,
    method = "GET",
    params,
    ...options
  } = {}) {
    const url = new URL(endpoint, this.baseUrl);
    if (params) {
      url.search = new URLSearchParams(params).toString();
    }
    headers = new fetch.Headers({
      accept: "*/*",
      "accept-encoding": "gzip,deflate",
      ...this.#headers,
      ...headers
    });
    const req = {
      ...options,
      method,
      timeout: this.#timeout
    };
    if (body instanceof URLSearchParams) {
      headers.set("Content-Type", "application/x-www-form-urlencoded");
      req.body = body;
    } else if (typeof body === "object") {
      headers.set("Content-Type", "application/json");
      req.body = JSON.stringify(body);
    }
    req.headers = headers;
    const res = await fetch(url, req);
    const contentType = res.headers.get("content-type") ?? "";
    const resBody = contentType.includes("application/json") ? await res.json() : await res.text();
    const meta = {
      headers: Object.fromEntries(res.headers.entries()),
      statusCode: res.status,
      statusText: res.statusText
    };
    if (!res.ok) {
      const opts = {
        ...meta,
        body: resBody
      };
      if (typeof resBody === "object") {
        const error = resBody.error;
        opts.type = error.type;
        opts.code = error.code;
      }
      throw new StremThruError(
        typeof resBody === "string" ? resBody : "message" in resBody.error ? `(${resBody.error.type}) ${resBody.error.message}` : JSON.stringify(resBody.error),
        opts
      );
    }
    return {
      data: resBody.data,
      meta
    };
  }
}

exports.StremThru = StremThru;
exports.StremThruError = StremThruError;
