import { RequestInit } from 'node-fetch';

type StoreMagnetStatus = "cached" | "downloaded" | "downloading" | "failed" | "invalid" | "processing" | "queued" | "unknown" | "uploading";
type StoreUserSubscriptionStatus = "expired" | "premium" | "trial";

type StremThruConfig = {
    auth?: string | {
        pass: string;
        user: string;
    } | {
        store: string;
        token: string;
    };
} & {
    baseUrl: string;
    clientIp?: string;
    timeout?: number;
    userAgent?: string;
};
type ResponseMeta = {
    headers: Record<string, string>;
    statusCode: number;
    statusText: string;
};
declare class StremThruStore {
    #private;
    constructor(client: StremThru, clientIp?: string);
    addMagnet({ clientIp, magnet, }: {
        clientIp?: string;
        magnet: string;
    }): Promise<{
        data: {
            added_at: string;
            files: Array<{
                index: number;
                link: string;
                name: string;
                path: string;
                size: number;
            }>;
            hash: string;
            id: string;
            magnet: string;
            name: string;
            status: StoreMagnetStatus;
        };
        meta: ResponseMeta;
    }>;
    checkMagnet(params: {
        magnet: string[];
        sid?: string;
    }): Promise<{
        data: {
            items: Array<{
                files: Array<{
                    index: number;
                    name: string;
                    size: number;
                }>;
                hash: string;
                magnet: string;
                status: StoreMagnetStatus;
            }>;
        };
        meta: ResponseMeta;
    }>;
    generateLink({ clientIp, link, }: {
        clientIp?: string;
        link: string;
    }): Promise<{
        data: {
            link: string;
        };
        meta: ResponseMeta;
    }>;
    getMagnet(magnetId: string): Promise<{
        data: {
            added_at: string;
            files: Array<{
                index: number;
                link: string;
                name: string;
                path: string;
                size: number;
            }>;
            hash: string;
            id: string;
            name: string;
            status: StoreMagnetStatus;
        };
        meta: ResponseMeta;
    }>;
    getUser(): Promise<{
        data: {
            email: string;
            id: string;
            subscription_status: StoreUserSubscriptionStatus;
        };
        meta: ResponseMeta;
    }>;
    listMagnets({ limit, offset, }: {
        limit?: number;
        offset?: number;
    }): Promise<{
        data: {
            items: Array<{
                added_at: string;
                hash: string;
                id: string;
                name: string;
                status: StoreMagnetStatus;
            }>;
            total_items: number;
        };
        meta: ResponseMeta;
    }>;
    removeMagnet(magnetId: string): Promise<{
        data: null;
        meta: ResponseMeta;
    }>;
}
declare class StremThru {
    #private;
    baseUrl: string;
    store: StremThruStore;
    constructor(config: StremThruConfig);
    health(): Promise<{
        data: {
            status: "ok";
        };
        meta: ResponseMeta;
    }>;
    request<T>(endpoint: string, { body, headers, method, params, ...options }?: Omit<RequestInit, "body"> & {
        body?: Record<string, unknown> | URLSearchParams;
        params?: Record<string, string | string[]> | URLSearchParams;
    }): Promise<{
        data: T;
        meta: ResponseMeta;
    }>;
}

type ErrorCode = "BAD_GATEWAY" | "BAD_REQUEST" | "CONFLICT" | "FORBIDDEN" | "GONE" | "INTERNAL_SERVER_ERROR" | "METHOD_NOT_ALLOWED" | "NOT_FOUND" | "NOT_IMPLEMENTED" | "PAYMENT_REQUIRED" | "PROXY_AUTHENTICATION_REQUIRED" | "SERVICE_UNAVAILABLE" | "STORE_LIMIT_EXCEEDED" | "STORE_MAGNET_INVALID" | "TOO_MANY_REQUESTS" | "UNAUTHORIZED" | "UNAVAILABLE_FOR_LEGAL_REASONS" | "UNKNOWN" | "UNPROCESSABLE_ENTITY" | "UNSUPPORTED_MEDIA_TYPE";
type ErrorType = "api_error" | "store_error" | "unknown_error" | "upstream_error";
declare class StremThruError extends Error {
    body?: unknown;
    code?: ErrorCode;
    headers: Record<string, string>;
    statusCode: number;
    statusText: string;
    type?: ErrorType;
    constructor(message: string, options: Pick<StremThruError, "body" | "code" | "headers" | "statusCode" | "statusText" | "type"> & {
        cause?: unknown;
    });
}

export { type ErrorCode, type ErrorType, type StoreMagnetStatus, type StoreUserSubscriptionStatus, StremThru, type StremThruConfig, StremThruError };
